@@ -7,9 +7,14 @@ class Instruments(Enum):
     COHERE = "cohere"
     PINECONE = "pinecone"
     CHROMA = "chroma"
+    GOOGLE_GENERATIVEAI = "google_generativeai"
     LANGCHAIN = "langchain"
+    MISTRAL = "mistral"
+    OLLAMA = "ollama"
     LLAMA_INDEX = "llama_index"
+    MILVUS = "milvus"
     TRANSFORMERS = "transformers"
+    TOGETHER = "together"
     REQUESTS = "requests"
     URLLIB3 = "urllib3"
     PYMYSQL = "pymysql"
@@ -18,3 +23,5 @@ class Instruments(Enum):
     VERTEXAI = "vertexai"
     WATSONX = "watsonx"
     WEAVIATE = "weaviate"
+    ALEPHALPHA = "alephalpha"
+    MARQO = "marqo"
