@@ -1,7 +1,6 @@
 import atexit
 import logging
 import os
-import importlib.util
 
 
 from colorama import Fore
@@ -29,8 +28,10 @@ from traceloop.sdk import Telemetry
 from traceloop.sdk.instruments import Instruments
 from traceloop.sdk.tracing.content_allow_list import ContentAllowList
 from traceloop.sdk.utils import is_notebook
+from traceloop.sdk.utils.package_check import is_package_installed
 from typing import Dict, Optional, Set
 
+
 TRACER_NAME = "traceloop.tracer"
 EXCLUDED_URLS = """
     iam.cloud.ibm.com,
@@ -46,7 +47,8 @@ EXCLUDED_URLS = """
     sentry.io,
     bedrock-runtime,
     googleapis.com,
-    githubusercontent.com"""
+    githubusercontent.com,
+    openaipublic.blob.core.windows.net"""
 
 
 class TracerWrapper(object):
@@ -54,6 +56,7 @@ class TracerWrapper(object):
     enable_content_tracing: bool = True
     endpoint: str = None
     headers: Dict[str, str] = {}
+    __tracer_provider: TracerProvider = None
 
     def __new__(
         cls,
@@ -70,9 +73,7 @@ class TracerWrapper(object):
                 return obj
 
             obj.__resource = Resource(attributes=TracerWrapper.resource_attributes)
-            obj.__tracer_provider: TracerProvider = init_tracer_provider(
-                resource=obj.__resource
-            )
+            obj.__tracer_provider = init_tracer_provider(resource=obj.__resource)
             if processor:
                 Telemetry().capture("tracer:init", {"processor": "custom"})
                 obj.__spans_processor: SpanProcessor = processor
@@ -158,6 +159,15 @@ class TracerWrapper(object):
                             print(Fore.RESET)
                         else:
                             instrument_set = True
+                    elif instrument == Instruments.GOOGLE_GENERATIVEAI:
+                        if not init_google_generativeai_instrumentor():
+                            print(
+                                Fore.RED
+                                + "Warning: Google Generative AI library does not exist."
+                            )
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
                     elif instrument == Instruments.LANGCHAIN:
                         if not init_langchain_instrumentor():
                             print(
@@ -166,6 +176,20 @@ class TracerWrapper(object):
                             print(Fore.RESET)
                         else:
                             instrument_set = True
+                    elif instrument == Instruments.MISTRAL:
+                        if not init_mistralai_instrumentor():
+                            print(
+                                Fore.RED + "Warning: MistralAI library does not exist."
+                            )
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
+                    elif instrument == Instruments.OLLAMA:
+                        if not init_ollama_instrumentor():
+                            print(Fore.RED + "Warning: Ollama library does not exist.")
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
                     elif instrument == Instruments.LLAMA_INDEX:
                         if not init_llama_index_instrumentor():
                             print(
@@ -174,6 +198,12 @@ class TracerWrapper(object):
                             print(Fore.RESET)
                         else:
                             instrument_set = True
+                    elif instrument == Instruments.MILVUS:
+                        if not init_milvus_instrumentor():
+                            print(Fore.RED + "Warning: Milvus library does not exist.")
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
                     elif instrument == Instruments.TRANSFORMERS:
                         if not init_transformers_instrumentor():
                             print(
@@ -183,6 +213,14 @@ class TracerWrapper(object):
                             print(Fore.RESET)
                         else:
                             instrument_set = True
+                    elif instrument == Instruments.TOGETHER:
+                        if not init_together_instrumentor():
+                            print(
+                                Fore.RED + "Warning: TogetherAI library does not exist."
+                            )
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
                     elif instrument == Instruments.REQUESTS:
                         if not init_requests_instrumentor():
                             print(
@@ -239,6 +277,21 @@ class TracerWrapper(object):
                             print(Fore.RESET)
                         else:
                             instrument_set = True
+                    elif instrument == Instruments.ALEPHALPHA:
+                        if not init_alephalpha_instrumentor():
+                            print(
+                                Fore.RED
+                                + "Warning: Aleph Alpha library does not exist."
+                            )
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
+                    elif instrument == Instruments.MARQO:
+                        if not init_marqo_instrumentor():
+                            print(Fore.RED + "Warning: marqo library does not exist.")
+                            print(Fore.RESET)
+                        else:
+                            instrument_set = True
 
                     else:
                         print(
@@ -280,16 +333,9 @@ class TracerWrapper(object):
         if entity_name is not None:
             span.set_attribute(SpanAttributes.TRACELOOP_ENTITY_NAME, entity_name)
 
-        correlation_id = get_value("correlation_id")
-        if correlation_id is not None:
-            span.set_attribute(SpanAttributes.TRACELOOP_CORRELATION_ID, correlation_id)
-
         association_properties = get_value("association_properties")
         if association_properties is not None:
-            for key, value in association_properties.items():
-                span.set_attribute(
-                    f"{SpanAttributes.TRACELOOP_ASSOCIATION_PROPERTIES}.{key}", value
-                )
+            _set_association_properties_attributes(span, association_properties)
 
             if not self.enable_content_tracing:
                 if self.__content_allow_list.is_allowed(association_properties):
@@ -298,27 +344,46 @@ class TracerWrapper(object):
                     attach(set_value("override_enable_content_tracing", False))
 
         if is_llm_span(span):
+            managed_prompt = get_value("managed_prompt")
+            if managed_prompt is not None:
+                span.set_attribute(
+                    SpanAttributes.TRACELOOP_PROMPT_MANAGED, managed_prompt
+                )
+
             prompt_key = get_value("prompt_key")
             if prompt_key is not None:
-                span.set_attribute("traceloop.prompt.key", prompt_key)
+                span.set_attribute(SpanAttributes.TRACELOOP_PROMPT_KEY, prompt_key)
 
             prompt_version = get_value("prompt_version")
             if prompt_version is not None:
-                span.set_attribute("traceloop.prompt.version", prompt_version)
+                span.set_attribute(
+                    SpanAttributes.TRACELOOP_PROMPT_VERSION, prompt_version
+                )
 
             prompt_version_name = get_value("prompt_version_name")
             if prompt_version_name is not None:
-                span.set_attribute("traceloop.prompt.version_name", prompt_version_name)
+                span.set_attribute(
+                    SpanAttributes.TRACELOOP_PROMPT_VERSION_NAME, prompt_version_name
+                )
 
             prompt_version_hash = get_value("prompt_version_hash")
             if prompt_version_hash is not None:
-                span.set_attribute("traceloop.prompt.version_hash", prompt_version_hash)
+                span.set_attribute(
+                    SpanAttributes.TRACELOOP_PROMPT_VERSION_HASH, prompt_version_hash
+                )
+
+            prompt_template = get_value("prompt_template")
+            if prompt_template is not None:
+                span.set_attribute(
+                    SpanAttributes.TRACELOOP_PROMPT_TEMPLATE, prompt_template
+                )
 
             prompt_template_variables = get_value("prompt_template_variables")
-            if prompt_version_hash is not None:
+            if prompt_template_variables is not None:
                 for key, value in prompt_template_variables.items():
                     span.set_attribute(
-                        f"traceloop.prompt.template_variables.{key}", value
+                        f"{SpanAttributes.TRACELOOP_PROMPT_TEMPLATE_VARIABLES}.{key}",
+                        value,
                     )
 
         # Call original on_start method if it exists in custom processor
@@ -362,6 +427,18 @@ class TracerWrapper(object):
 def set_association_properties(properties: dict) -> None:
     attach(set_value("association_properties", properties))
 
+    # Attach association properties to the current span, if it's a workflow or a task
+    span = trace.get_current_span()
+    if get_value("workflow_name") is not None or get_value("entity_name") is not None:
+        _set_association_properties_attributes(span, properties)
+
+
+def _set_association_properties_attributes(span, properties: dict) -> None:
+    for key, value in properties.items():
+        span.set_attribute(
+            f"{SpanAttributes.TRACELOOP_ASSOCIATION_PROPERTIES}.{key}", value
+        )
+
 
 def set_workflow_name(workflow_name: str) -> None:
     attach(set_value("workflow_name", workflow_name))
@@ -379,13 +456,14 @@ def get_chained_entity_name(entity_name: str) -> str:
         return f"{parent}.{entity_name}"
 
 
-def set_prompt_tracing_context(
+def set_managed_prompt_tracing_context(
     key: str,
     version: int,
     version_name: str,
     version_hash: str,
     template_variables: dict,
 ) -> None:
+    attach(set_value("managed_prompt", True))
     attach(set_value("prompt_key", key))
     attach(set_value("prompt_version", version))
     attach(set_value("prompt_version_name", version_name))
@@ -393,6 +471,15 @@ def set_prompt_tracing_context(
     attach(set_value("prompt_template_variables", template_variables))
 
 
+def set_external_prompt_tracing_context(
+    template: str, variables: dict, version: int
+) -> None:
+    attach(set_value("managed_prompt", False))
+    attach(set_value("prompt_version", version))
+    attach(set_value("prompt_template", template))
+    attach(set_value("prompt_template_variables", variables))
+
+
 def is_llm_span(span) -> bool:
     return span.attributes.get(SpanAttributes.LLM_REQUEST_TYPE) is not None
 
@@ -429,10 +516,15 @@ def init_instrumentations(should_enrich_metrics: bool):
     init_pinecone_instrumentor()
     init_qdrant_instrumentor()
     init_chroma_instrumentor()
+    init_google_generativeai_instrumentor()
     init_haystack_instrumentor()
     init_langchain_instrumentor()
+    init_mistralai_instrumentor()
+    init_ollama_instrumentor()
     init_llama_index_instrumentor()
+    init_milvus_instrumentor()
     init_transformers_instrumentor()
+    init_together_instrumentor()
     init_requests_instrumentor()
     init_urllib3_instrumentor()
     init_pymysql_instrumentor()
@@ -441,230 +533,475 @@ def init_instrumentations(should_enrich_metrics: bool):
     init_vertexai_instrumentor()
     init_watsonx_instrumentor()
     init_weaviate_instrumentor()
+    init_alephalpha_instrumentor()
+    init_marqo_instrumentor()
 
 
 def init_openai_instrumentor(should_enrich_metrics: bool):
-    if importlib.util.find_spec("openai") is not None:
-        Telemetry().capture("instrumentation:openai:init")
-        from opentelemetry.instrumentation.openai import OpenAIInstrumentor
+    try:
+        if is_package_installed("openai"):
+            Telemetry().capture("instrumentation:openai:init")
+            from opentelemetry.instrumentation.openai import OpenAIInstrumentor
 
-        instrumentor = OpenAIInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-            enrich_assistant=should_enrich_metrics,
-            enrich_token_usage=should_enrich_metrics,
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = OpenAIInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+                enrich_assistant=should_enrich_metrics,
+                enrich_token_usage=should_enrich_metrics,
+                get_common_metrics_attributes=metrics_common_attributes,
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+
+    except Exception as e:
+        logging.error(f"Error initializing OpenAI instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_anthropic_instrumentor(should_enrich_metrics: bool):
-    if importlib.util.find_spec("anthropic") is not None:
-        Telemetry().capture("instrumentation:anthropic:init")
-        from opentelemetry.instrumentation.anthropic import AnthropicInstrumentor
+    try:
+        if is_package_installed("anthropic"):
+            Telemetry().capture("instrumentation:anthropic:init")
+            from opentelemetry.instrumentation.anthropic import AnthropicInstrumentor
 
-        instrumentor = AnthropicInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-            enrich_token_usage=should_enrich_metrics,
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = AnthropicInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+                enrich_token_usage=should_enrich_metrics,
+                get_common_metrics_attributes=metrics_common_attributes,
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Anthropic instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_cohere_instrumentor():
-    if importlib.util.find_spec("cohere") is not None:
-        Telemetry().capture("instrumentation:cohere:init")
-        from opentelemetry.instrumentation.cohere import CohereInstrumentor
+    try:
+        if is_package_installed("cohere"):
+            Telemetry().capture("instrumentation:cohere:init")
+            from opentelemetry.instrumentation.cohere import CohereInstrumentor
 
-        instrumentor = CohereInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = CohereInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Cohere instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_pinecone_instrumentor():
-    if importlib.util.find_spec("pinecone") is not None:
-        Telemetry().capture("instrumentation:pinecone:init")
-        from opentelemetry.instrumentation.pinecone import PineconeInstrumentor
+    try:
+        if is_package_installed("pinecone"):
+            Telemetry().capture("instrumentation:pinecone:init")
+            from opentelemetry.instrumentation.pinecone import PineconeInstrumentor
 
-        instrumentor = PineconeInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = PineconeInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Pinecone instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_qdrant_instrumentor():
-    if importlib.util.find_spec("qdrant_client") is not None:
-        Telemetry().capture("instrumentation:qdrant:init")
-        from opentelemetry.instrumentation.qdrant import QdrantInstrumentor
+    try:
+        if is_package_installed("qdrant_client"):
+            Telemetry().capture("instrumentation:qdrant:init")
+            from opentelemetry.instrumentation.qdrant import QdrantInstrumentor
 
-        instrumentor = QdrantInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
+            instrumentor = QdrantInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+    except Exception as e:
+        logging.error(f"Error initializing Qdrant instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_chroma_instrumentor():
-    if importlib.util.find_spec("chromadb") is not None:
-        Telemetry().capture("instrumentation:chromadb:init")
-        from opentelemetry.instrumentation.chromadb import ChromaInstrumentor
+    try:
+        if is_package_installed("chromadb"):
+            Telemetry().capture("instrumentation:chromadb:init")
+            from opentelemetry.instrumentation.chromadb import ChromaInstrumentor
 
-        instrumentor = ChromaInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = ChromaInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Chroma instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_google_generativeai_instrumentor():
+    try:
+        if is_package_installed("google.generativeai"):
+            Telemetry().capture("instrumentation:gemini:init")
+            from opentelemetry.instrumentation.google_generativeai import (
+                GoogleGenerativeAiInstrumentor,
+            )
+
+            instrumentor = GoogleGenerativeAiInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Gemini instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_haystack_instrumentor():
-    if importlib.util.find_spec("haystack") is not None:
-        Telemetry().capture("instrumentation:haystack:init")
-        from opentelemetry.instrumentation.haystack import HaystackInstrumentor
+    try:
+        if is_package_installed("haystack"):
+            Telemetry().capture("instrumentation:haystack:init")
+            from opentelemetry.instrumentation.haystack import HaystackInstrumentor
 
-        instrumentor = HaystackInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = HaystackInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Haystack instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_langchain_instrumentor():
-    if importlib.util.find_spec("langchain") is not None:
-        Telemetry().capture("instrumentation:langchain:init")
-        from opentelemetry.instrumentation.langchain import LangchainInstrumentor
+    try:
+        if is_package_installed("langchain"):
+            Telemetry().capture("instrumentation:langchain:init")
+            from opentelemetry.instrumentation.langchain import LangchainInstrumentor
 
-        instrumentor = LangchainInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = LangchainInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing LangChain instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_mistralai_instrumentor():
+    try:
+        if is_package_installed("mistralai"):
+            Telemetry().capture("instrumentation:mistralai:init")
+            from opentelemetry.instrumentation.mistralai import MistralAiInstrumentor
+
+            instrumentor = MistralAiInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing MistralAI instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_ollama_instrumentor():
+    try:
+        if is_package_installed("ollama"):
+            Telemetry().capture("instrumentation:ollama:init")
+            from opentelemetry.instrumentation.ollama import OllamaInstrumentor
+
+            instrumentor = OllamaInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Ollama instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_transformers_instrumentor():
-    if importlib.util.find_spec("transformers") is not None:
-        Telemetry().capture("instrumentation:transformers:init")
-        from opentelemetry.instrumentation.transformers import TransformersInstrumentor
+    try:
+        if is_package_installed("transformers"):
+            Telemetry().capture("instrumentation:transformers:init")
+            from opentelemetry.instrumentation.transformers import (
+                TransformersInstrumentor,
+            )
 
-        instrumentor = TransformersInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = TransformersInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Transformers instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_together_instrumentor():
+    try:
+        if is_package_installed("together"):
+            Telemetry().capture("instrumentation:together:init")
+            from opentelemetry.instrumentation.together import TogetherAiInstrumentor
+
+            instrumentor = TogetherAiInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing TogetherAI instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_llama_index_instrumentor():
-    if importlib.util.find_spec("llama_index") is not None:
-        Telemetry().capture("instrumentation:llamaindex:init")
-        from opentelemetry.instrumentation.llamaindex import LlamaIndexInstrumentor
+    try:
+        if is_package_installed("llama-index") or is_package_installed("llama_index"):
+            Telemetry().capture("instrumentation:llamaindex:init")
+            from opentelemetry.instrumentation.llamaindex import LlamaIndexInstrumentor
 
-        instrumentor = LlamaIndexInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = LlamaIndexInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing LlamaIndex instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_milvus_instrumentor():
+    try:
+        if is_package_installed("pymilvus"):
+            Telemetry().capture("instrumentation:milvus:init")
+            from opentelemetry.instrumentation.milvus import MilvusInstrumentor
+
+            instrumentor = MilvusInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Milvus instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_requests_instrumentor():
-    if importlib.util.find_spec("requests") is not None:
-        from opentelemetry.instrumentation.requests import RequestsInstrumentor
+    try:
+        if is_package_installed("requests"):
+            from opentelemetry.instrumentation.requests import RequestsInstrumentor
 
-        instrumentor = RequestsInstrumentor()
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument(excluded_urls=EXCLUDED_URLS)
-    return True
+            instrumentor = RequestsInstrumentor()
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument(excluded_urls=EXCLUDED_URLS)
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Requests instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_urllib3_instrumentor():
-    if importlib.util.find_spec("urllib3") is not None:
-        from opentelemetry.instrumentation.urllib3 import URLLib3Instrumentor
+    try:
+        if is_package_installed("urllib3"):
+            from opentelemetry.instrumentation.urllib3 import URLLib3Instrumentor
 
-        instrumentor = URLLib3Instrumentor()
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument(excluded_urls=EXCLUDED_URLS)
-    return True
+            instrumentor = URLLib3Instrumentor()
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument(excluded_urls=EXCLUDED_URLS)
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing urllib3 instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_pymysql_instrumentor():
-    if importlib.util.find_spec("sqlalchemy") is not None:
-        from opentelemetry.instrumentation.sqlalchemy import SQLAlchemyInstrumentor
+    try:
+        if is_package_installed("sqlalchemy"):
+            from opentelemetry.instrumentation.sqlalchemy import SQLAlchemyInstrumentor
 
-        instrumentor = SQLAlchemyInstrumentor()
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = SQLAlchemyInstrumentor()
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing SQLAlchemy instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_bedrock_instrumentor(should_enrich_metrics: bool):
-    if importlib.util.find_spec("boto3") is not None:
-        from opentelemetry.instrumentation.bedrock import BedrockInstrumentor
+    try:
+        if is_package_installed("boto3"):
+            from opentelemetry.instrumentation.bedrock import BedrockInstrumentor
 
-        instrumentor = BedrockInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-            enrich_token_usage=should_enrich_metrics,
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = BedrockInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+                enrich_token_usage=should_enrich_metrics,
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Bedrock instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_replicate_instrumentor():
-    if importlib.util.find_spec("replicate") is not None:
-        Telemetry().capture("instrumentation:replicate:init")
-        from opentelemetry.instrumentation.replicate import ReplicateInstrumentor
+    try:
+        if is_package_installed("replicate"):
+            Telemetry().capture("instrumentation:replicate:init")
+            from opentelemetry.instrumentation.replicate import ReplicateInstrumentor
 
-        instrumentor = ReplicateInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = ReplicateInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Replicate instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_vertexai_instrumentor():
-    if importlib.util.find_spec("vertexai") is not None:
-        Telemetry().capture("instrumentation:vertexai:init")
-        from opentelemetry.instrumentation.vertexai import VertexAIInstrumentor
+    try:
+        if is_package_installed("vertexai"):
+            Telemetry().capture("instrumentation:vertexai:init")
+            from opentelemetry.instrumentation.vertexai import VertexAIInstrumentor
 
-        instrumentor = VertexAIInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = VertexAIInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.warning(f"Error initializing Vertex AI instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_watsonx_instrumentor():
-    if importlib.util.find_spec("ibm_watson_machine_learning") is not None:
-        Telemetry().capture("instrumentation:watsonx:init")
-        from opentelemetry.instrumentation.watsonx import WatsonxInstrumentor
+    try:
+        if is_package_installed("ibm_watson_machine_learning"):
+            Telemetry().capture("instrumentation:watsonx:init")
+            from opentelemetry.instrumentation.watsonx import WatsonxInstrumentor
 
-        instrumentor = WatsonxInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = WatsonxInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.warning(f"Error initializing Watsonx instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
 
 
 def init_weaviate_instrumentor():
-    if importlib.util.find_spec("weaviate") is not None:
-        Telemetry().capture("instrumentation:weaviate:init")
-        from opentelemetry.instrumentation.weaviate import WeaviateInstrumentor
+    try:
+        if is_package_installed("weaviate"):
+            Telemetry().capture("instrumentation:weaviate:init")
+            from opentelemetry.instrumentation.weaviate import WeaviateInstrumentor
 
-        instrumentor = WeaviateInstrumentor(
-            exception_logger=lambda e: Telemetry().log_exception(e),
-        )
-        if not instrumentor.is_instrumented_by_opentelemetry:
-            instrumentor.instrument()
-    return True
+            instrumentor = WeaviateInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.warning(f"Error initializing Weaviate instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_alephalpha_instrumentor():
+    try:
+        if is_package_installed("aleph_alpha_client"):
+            Telemetry().capture("instrumentation:alephalpha:init")
+            from opentelemetry.instrumentation.alephalpha import AlephAlphaInstrumentor
+
+            instrumentor = AlephAlphaInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Aleph Alpha instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_marqo_instrumentor():
+    try:
+        if is_package_installed("marqo"):
+            Telemetry().capture("instrumentation:marqo:init")
+            from opentelemetry.instrumentation.marqo import MarqoInstrumentor
+
+            instrumentor = MarqoInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing marqo instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def metrics_common_attributes():
+    common_attributes = {}
+    workflow_name = get_value("workflow_name")
+    if workflow_name is not None:
+        common_attributes[SpanAttributes.TRACELOOP_WORKFLOW_NAME] = workflow_name
+
+    entity_name = get_value("entity_name")
+    if entity_name is not None:
+        common_attributes[SpanAttributes.TRACELOOP_ENTITY_NAME] = entity_name
+
+    association_properties = get_value("association_properties")
+    if association_properties is not None:
+        for key, value in association_properties.items():
+            common_attributes[
+                f"{SpanAttributes.TRACELOOP_ASSOCIATION_PROPERTIES}.{key}"
+            ] = value
+
+    return common_attributes
